@@ -647,8 +647,10 @@ char  *AFS::get_execute_dir()
 
 
 //Return 0 if init failed.
-int    AFS::init_support(char *fname)
+int    AFS::init_support(char *fname, char* subdir)
 {
+  int idx;
+  xncdprintf(("AFS::ini arcfile --> %s <--\n",fname));
   strcpy(arcname, fname);
   update_on_exit = 0;
   sublevel = 0;
@@ -661,7 +663,7 @@ int    AFS::init_support(char *fname)
   if (dont_construct_path == 0)
     {
 // *INDENT-OFF*        
-      ::getcwd(fullarcname, 800);
+      ::getcwd(fullarcname, 1024);
 // *INDENT-ON*        
 
       strcat(fullarcname, "/");
@@ -687,7 +689,21 @@ int    AFS::init_support(char *fname)
   sublevel = 0;
   need_change_vfs = 0;
   strcpy(curdir, ".");
-  to_dir[0] = 0;
+  if(subdir)
+  {
+    if(*subdir=='/')
+      subdir++;
+    strcpy(to_dir,subdir);
+    /*    idx=0;
+    if(subdir[idx]!='.')
+      curdir[idx++]='.';
+    if(subdir[0]!='/' && idx==0)
+      curdir[idx++]='/';
+    strcpy(curdir+idx,subdir);
+    */
+  }
+  else
+    to_dir[0] = 0;
   return reread();
 }
 
@@ -1202,7 +1218,7 @@ int    AFS::reread()
   char  *fname;
   char   attrchr[20];
   char  *w;
-  char   strtmp[L_MAXPATH];
+  char   strtmp[2*L_MAXPATH];
   char   lbuf1[L_MAXPATH];
   char   lbuf2[L_MAXPATH];
   int    fsiz;
@@ -1212,8 +1228,11 @@ int    AFS::reread()
   Namer *namer=NULL,*n,*n2;
   DTree *dt;
   
-  
-  sprintf(strtmp, sp->list, fullarcname);
+  char tmp_fulldir[L_MAXPATH];
+
+  quote_path(tmp_fulldir,fullarcname);
+  sprintf(strtmp, sp->list, tmp_fulldir);
+
   fp2 = fopen(listpath, "w");
   if (fp2 == NULL)
     {
@@ -1356,154 +1375,155 @@ int    AFS::direc(char *dir_ch)
   char *to_dir=this->to_dir;
   int linkcoun=0,l=strlen(to_dir);
   
-  
+  xncdprintf(("AFS:direc --> to_dir=[%s] dir=[%s] sublevel=%d\n",to_dir,dir_ch,sublevel));
   while(l>2 && strncmp(to_dir,"..",2)==0)
   {
-          sublevel--;
-          if(sublevel<0)
-          {
-                  sublevel=0;
-                  vfs_error("AFS", "Can't follow directory (outside archive)!");
-                  return 255;
-          }
-          upper_path(curdir);
-          to_dir+=3;
+    sublevel--;
+    if(sublevel<0)
+    {
+      sublevel=0;
+      vfs_error("AFS", "Can't follow directory (outside archive)!");
+      return 255;
+    }
+    upper_path(curdir);
+    to_dir+=3;
   }
   
   if (strcmp(to_dir, "..") == 0)
+  {
+    if (sublevel == 0)
     {
-      if (sublevel == 0)
-        {
-          need_change_vfs = 1;
-          return 0;
-        }
-      else
-        {
-          sublevel--;
-          upper_path(curdir);
-        };
+      need_change_vfs = 1;
+      return 0;
     }
+    else
+    {
+      sublevel--;
+      upper_path(curdir);
+    };
+  }
   else if (find_content(to_dir))
+  {
+    sublevel++;
+    add_path_content(curdir, to_dir);
+  } else        //If we here then we doing cd not to subdir but to subdir/subdir...
+  {
+    char *s=to_dir;
+    if(*s)
     {
-      sublevel++;
-      add_path_content(curdir, to_dir);
-    } else        //If we here then we doing cd not to subdir but to subdir/subdir...
-    {
-            char *s=to_dir;
-            if(*s)
-            {
-                    do
-                    {
-                            s=get_first_content(s,strtmp);
-                            add_path_content(curdir, strtmp);
-                            sublevel++;
-                    }while(s);
-            }
+      do
+      {
+	s=get_first_content(s,strtmp);
+	add_path_content(curdir, strtmp);
+	sublevel++;
+      }while(s);
     }
+  }
   this->to_dir[0] = 0;
+  xncdprintf(("AFS:direc **> to_dir=[%s] dir=[%s] sublevel=%d\n",to_dir,curdir,sublevel));
   if (need_reread)
     reread();
   fp = fopen(listpath, "r");
   if (fp == NULL)
-    {
-      vfs_error("AFS", "Can't access TMP dir");
-      need_change_vfs = 1;
-      return 0;
-    }
+  {
+    vfs_error("AFS", "Can't access TMP dir");
+    need_change_vfs = 1;
+    return 0;
+  }
   VFS::delete_vfs_list();
   while (fgets(strtmp,sizeof(strtmp),fp))
-    {
-      bmark->animate();
-      sscanf(strtmp, "%d %X %d %d %[^\n]\n", &coun, &mode, &fsiz, &t, name);
-      if (coun == sublevel && strcmp(name, "./.") != 0)
-        if (is_in_this_dir(name))
-          {
-            if (hide_hidden && name[2] == '.')
-              continue;
-            ol = (FList*)new_alloc(S_FList);
-            if(mode & S_IFLNK)
-            {
-                    breakdown_link(name, lbuf1, lbuf2);
-                    ol->init(get_lastname(lbuf1), mode, fsiz, getuid(), getgid(), t);
-                    if((mode & S_IFDIR)==S_IFDIR)
-                    {
-                            ol->link=(FList*)new_alloc(S_FList);
-                            ol->link->init(lbuf2,ol->mode,0,0,0,t);
-                    } else
-                    {
-                            linkcoun++;
-                            n=new Namer(lbuf2,ol);
-                            n->next=namer;
-                            namer=n;
-                    }
-            } else
-                    ol->init(get_lastname(name), mode, fsiz, getuid(), getgid(), t);
-            ol->options &= ~S_SELECT;
-            if (mode & S_IFDIR)
-              dl.add(ol);
-            else
-              {
-                fl.add(ol);
-                if (ext_find(ol->name) != -1)
-                  ol->options |= S_EXTEND;
-                else if (is_it_afs_file(ol->name))
-                  ol->options |= S_EXTEND;
-              }
-          }
-    }
+  {
+    bmark->animate();
+    sscanf(strtmp, "%d %X %d %d %[^\n]\n", &coun, &mode, &fsiz, &t, name);
+    if (coun == sublevel && strcmp(name, "./.") != 0)
+      if (is_in_this_dir(name))
+      {
+	if (hide_hidden && name[2] == '.')
+	  continue;
+	ol = (FList*)new_alloc(S_FList);
+	if(mode & S_IFLNK)
+	{
+	  breakdown_link(name, lbuf1, lbuf2);
+	  ol->init(get_lastname(lbuf1), mode, fsiz, getuid(), getgid(), t);
+	  if((mode & S_IFDIR)==S_IFDIR)
+	  {
+	    ol->link=(FList*)new_alloc(S_FList);
+	    ol->link->init(lbuf2,ol->mode,0,0,0,t);
+	  } else
+	  {
+	    linkcoun++;
+	    n=new Namer(lbuf2,ol);
+	    n->next=namer;
+	    namer=n;
+	  }
+	} else
+	  ol->init(get_lastname(name), mode, fsiz, getuid(), getgid(), t);
+	ol->options &= ~S_SELECT;
+	if (mode & S_IFDIR)
+	  dl.add(ol);
+	else
+	{
+	  fl.add(ol);
+	  if (ext_find(ol->name) != -1)
+	    ol->options |= S_EXTEND;
+	  else if (is_it_afs_file(ol->name))
+	    ol->options |= S_EXTEND;
+	}
+      }
+  }
   fclose(fp);
 
   if(linkcoun)
   {                        //Doing post action linking
-        n=namer;
-        struct stat st;
-        while(n)
-        {
-              ol=(FList*)new_alloc(S_FList);
-              if(flist_find_by_name(fl.next,n->name,&st))
-              {
-                      ol->init(n->name, st.st_mode, st.st_size, st.st_uid, st.st_gid, st.st_mtime);
-              }
-              else
-                if(n->o->mode & S_IFDIR)
-                {
-                      ol->init(n->name,n->o->mode,0,0,0,n->o->time);
-                      st.st_mode=ol->mode;
-                      st.st_mtime=ol->time;
-                }
-                else
-                {
-                      ol->init(n->name,0755,7,0,0,st.st_mtime);
-                      st.st_mode=ol->mode;
-                }
-              n->o->link=ol;
-              n->o->size=ol->size;
-              ol=n->o;
-              if((ol->mode & S_IFDIR)!=S_IFDIR && (st.st_mode & S_IFDIR)==S_IFDIR)
-              {        //We are in wrong list. We must be inside dl but now we in fl.
-                      ol->mode|=S_IFDIR;
-                      if(ol->prev)
-                              ol->prev->next=ol->next;
-                      if(ol->next)
-                              ol->next->prev=ol->prev;
-                      ol->next=ol->prev=NULL;
-                      dl.add(ol);
-              } else
-                      if((ol->mode & S_IFDIR)==S_IFDIR && (st.st_mode & S_IFDIR)!=S_IFDIR)
-                      {        //We are in wrong list. We must be inside fl but now we in dl.
-                              ol->mode&=~S_IFDIR;
-                              if(ol->prev)
-                                      ol->prev->next=ol->next;
-                              if(ol->next)
-                                      ol->next->prev=ol->prev;
-                              ol->next=ol->prev=NULL;
-                              fl.add(ol);
-                      }              
-              ol->mode=(ol->mode & ~0777) | (st.st_mode & 0777);
-              n=n->next;
-              delete namer;
-              namer=n;
-        }
+    n=namer;
+    struct stat st;
+    while(n)
+    {
+      ol=(FList*)new_alloc(S_FList);
+      if(flist_find_by_name(fl.next,n->name,&st))
+      {
+	ol->init(n->name, st.st_mode, st.st_size, st.st_uid, st.st_gid, st.st_mtime);
+      }
+      else
+	if(n->o->mode & S_IFDIR)
+	{
+	  ol->init(n->name,n->o->mode,0,0,0,n->o->time);
+	  st.st_mode=ol->mode;
+	  st.st_mtime=ol->time;
+	}
+	else
+	{
+	  ol->init(n->name,0755,7,0,0,st.st_mtime);
+	  st.st_mode=ol->mode;
+	}
+      n->o->link=ol;
+      n->o->size=ol->size;
+      ol=n->o;
+      if((ol->mode & S_IFDIR)!=S_IFDIR && (st.st_mode & S_IFDIR)==S_IFDIR)
+      {        //We are in wrong list. We must be inside dl but now we in fl.
+	ol->mode|=S_IFDIR;
+	if(ol->prev)
+	  ol->prev->next=ol->next;
+	if(ol->next)
+	  ol->next->prev=ol->prev;
+	ol->next=ol->prev=NULL;
+	dl.add(ol);
+      } else
+	if((ol->mode & S_IFDIR)==S_IFDIR && (st.st_mode & S_IFDIR)!=S_IFDIR)
+	{        //We are in wrong list. We must be inside fl but now we in dl.
+	  ol->mode&=~S_IFDIR;
+	  if(ol->prev)
+	    ol->prev->next=ol->next;
+	  if(ol->next)
+	    ol->next->prev=ol->prev;
+	  ol->next=ol->prev=NULL;
+	  fl.add(ol);
+	}              
+      ol->mode=(ol->mode & ~0777) | (st.st_mode & 0777);
+      n=n->next;
+      delete namer;
+      namer=n;
+    }
   }
   
   bmark->animate();
