@@ -4,4 +4,5 @@
 extern void   magic_scanner();
 extern int    magic_need_scan;
 
-#endif
\ No newline at end of file
+#endif/* ------------ End of file -------------- */
+
