@@ -327,7 +327,7 @@ void   msg(char *meshead, char *mes, int col, FList * o)
 KEY   *wwk1, *wwk2, *wwk3;
 Text  *wt1, *wt2, *wt3, *wt4, *wt5, *wt02;
 Panel *pan, *pan2;
-int    num;
+ManFuncs    num;
 int    attrmode;
 char   wpath[256] = "Path:  none";
 char   w_to_path[256] = "Path:  none";
@@ -359,7 +359,7 @@ int    sigkills[10] =
 {SIGKILL, SIGSTOP, SIGCONT, SIGINT, SIGHUP, SIGTERM, SIGALRM, SIGUSR1, SIGUSR2, SIGKILL};
 Sw_panel *wot, *wow, *wgr, *swkill = NULL;
 
-int    meswinnum;
+ManFuncs    meswinnum;
 void   meswinok()
 {
   wwk1->hide();
@@ -377,7 +377,7 @@ void   meswinok()
   XFlush(disp);
 }
 
-void   create_meswin(char *head, char *te, int n)
+void   create_meswin(char *head, char *te, ManFuncs n)
 {
   if (ww == NULL)
     {
@@ -550,7 +550,7 @@ void   atclear()
   panel->reread();
 }
 
-void   attrib(FList * cur, int inum)
+void   attrib(FList * cur, ManFuncs inum)
 {
   int    i, mask = S_IRUSR;
   if (ww == NULL)
@@ -768,7 +768,7 @@ void   canquery()
   ww = NULL;
 }
 
-void   initquery(char *head, Lister * p1, VFS * vfs, int inum)
+void   initquery(char *head, Lister * p1, VFS * vfs, ManFuncs inum)
 {
   int    tl;
   int    i, mask = S_IRUSR;
@@ -861,7 +861,7 @@ void   initquery(char *head, Lister * p1, VFS * vfs, int inum)
     }
 }
 
-void   init_copymove_query(char *head, Lister * p1, VFS * vfs, int inum)
+void   init_copymove_query(char *head, Lister * p1, VFS * vfs, ManFuncs inum)
 {
   int    tl;
   char tmpst[20];
@@ -1070,7 +1070,7 @@ void   fchowncan()
 void   fchownok()
 {
   fchowncan();
-  panel->funcs(20);
+  panel->funcs(FCHOWN);
 }
 
 int    panusr(int, char *m)
@@ -1251,6 +1251,9 @@ void conf_can()
         confok->hide();
         confcan->hide();
         confwp->hide();
+	confin1->hide();
+	confwmin1->hide();
+	confin_mag->hide();
         confw->hide();
         
         delete confcan;
@@ -1681,6 +1684,14 @@ void ftp_can()
   ftpcan->hide();
   ftpwp->hide();
   ftpw->hide();
+
+  ftpin1->hide();
+  ftpin2->hide();
+  ftpin3->hide();
+  ftpin4->hide();
+  ftpin5->hide();
+  ftpin6->hide();
+
   delete ftpok;
   delete ftpcan;
   delete ftpwp;
@@ -1901,10 +1912,15 @@ int ftp_list_to_host(int idx, char*)
       ftpsw->sw=0;
 
   proxsw->sw=ftprec.bool_prox;
-  
   return 0;
 }
 
+int ftp_list_to_host_and_go(int idx, char* ptr)
+{
+  ftp_list_to_host(idx,ptr);
+  ftp_ok();
+  return 0;
+}
 
 
 void   ftp_update_rec()
@@ -1928,7 +1944,7 @@ void   ftp_update_rec()
 }
 
 
-void   new_ftp_prompt(Lister *l1, int inum)
+void   new_ftp_prompt(Lister *l1, ManFuncs inum)
 {
   if (ftpw == NULL)
     {
@@ -1943,7 +1959,8 @@ void   new_ftp_prompt(Lister *l1, int inum)
       ftpcan->set_iname("FTPCAN");
       ftpwp->set_iname("FTPPAGE");
       ftpt1  = guiplugin->new_Text(10, 50, "FTP Sessions:", 5);
-      pan    = guiplugin->new_Panel(10, 60, 360, 220, hosts_list, max_hosts, 1, ftp_list_to_host);
+      pan    = guiplugin->new_Panel(10, 60, 360, 220, hosts_list, max_hosts, 1, 
+				    ftp_list_to_host_and_go);
       
       if(max_hosts)
           ftprec=*((FTPCFG_S*)hosts_list[0]);
@@ -2023,7 +2040,7 @@ void   new_ftp_prompt(Lister *l1, int inum)
       proxsw->init(ftpwp->w);
       pan->init(ftpwp->w);
       pan->escfunc(ftp_can);
-      pan->set_every_move_to(1);
+      pan->set_every_move_to(ftp_list_to_host);
       ftpin1->set_escapefunc(ftp_can);
       ftpin2->set_escapefunc(ftp_can);
       ftpin3->set_escapefunc(ftp_can);
@@ -2127,6 +2144,7 @@ void   new_ftp_prompt(Lister *l1, int inum)
       ftpwp->show();
       ftpok->show();
       ftpcan->show();
+      pan->set_selffocus();
     }
 }
 
@@ -2143,7 +2161,7 @@ void   set_dbuf(VFS* vf)
         }
 }
 
-void   initdialog(char *head, Lister * l1, VFS * vf, int inum)
+void   initdialog(char *head, Lister * l1, VFS * vf, ManFuncs inum)
 {
   char tmpst[20];
   if (dw == NULL)
@@ -2205,7 +2223,7 @@ void   initdialog(char *head, Lister * l1, VFS * vf, int inum)
     }
 }
 
-void   newtextfile(char *head, Lister * l1, int inum)
+void   newtextfile(char *head, Lister * l1, ManFuncs inum, int showtext)
 {
   char tmpst[20];
   if (dw == NULL)
@@ -2235,8 +2253,10 @@ void   newtextfile(char *head, Lister * l1, int inum)
       dcan->link(din);
       dbuf[0] = 0;
       din->setbuf(dbuf, 255);
-      if(inum!=11)        //Skip man pages
+      if(inum!=FMANPAGE)        //Skip man pages
          din->bool_expand=YES;
+      if(!showtext)
+	din->set_passwd_mode();
       dw->show();
       din->show();
       dok->show();
@@ -2327,7 +2347,7 @@ int    panmnt(int i, char *m)
   pancan();
   tt_printf("umount %s\n", m);
   strcpy(mntmnt, m);
-  create_meswin("Remounting", "Change DEVICE NOW", 22);
+  create_meswin("Remounting", "Change DEVICE NOW", FMOUNT);
   return 0;
 }
 
@@ -2482,7 +2502,7 @@ void   menupanel(int (*prfunc) (int, char *))
     }
 }
 
-void   selectmask(Lister * l1, int inum)
+void   selectmask(Lister * l1, ManFuncs inum)
 {
   if (dw == NULL)
     {
