@@ -90,6 +90,8 @@
  * /dev/tty belongs to the group "tty" and not "system" or "wheel"
  */
 
+int rxvt_ttyfd=-1;
+
 extern int main_show;
 int vt_show=1;
 int focus_fl=0;
@@ -126,8 +128,12 @@ static short    changettyowner = 1;
 
 #if defined (__svr4__)
 # include <sys/resource.h>        /* for struct rlimit */
-# include <sys/stropts.h>        /* for I_PUSH */
-# define _NEW_TTY_CTRL                /* to get proper defines in <termios.h> */
+
+#if defined(PTYS_ARE_PTMX) && !defined(__CYGWIN32__)
+# include <sys/stropts.h>      /* for I_PUSH */
+#endif
+
+#define _NEW_TTY_CTRL                /* to get proper defines in <termios.h> */
 #endif
 
 #ifdef HAVE_SYS_SELECT_H
@@ -625,6 +631,15 @@ get_pty(void)
     int             fd;
     char           *ptydev;
 
+#ifdef PTYS_ARE_OPENPTY
+    char            tty_name[sizeof "/dev/pts/????\0"];
+
+    if (openpty(&fd, &rxvt_ttyfd, tty_name, NULL, NULL) != -1) {
+        ttydev = strdup(tty_name);
+        return fd;
+    }
+#endif
+
 #ifdef PTYS_ARE__GETPTY
     if ((ptydev = ttydev = _getpty(&fd, O_RDWR | O_NDELAY, 0622, 0)) != NULL)
 	goto Found;
@@ -764,25 +779,35 @@ get_tty(void)
     print_error("(%s: line %d): PID = %d\n", __FILE__, __LINE__, pid);
 # endif
 #endif				/* ultrix */
-
-    if ((fd = open(ttydev, O_RDWR)) < 0
-	&& (fd = open("/dev/tty", O_RDWR)) < 0) {
+    
+    if(rxvt_ttyfd==-1) /* we are already open tty*/
+    {
+      if ((fd = open(ttydev, O_RDWR)) < 0
+	  && (fd = open("/dev/tty", O_RDWR)) < 0) 
+      {
 	print_error("can't open slave tty %s", ttydev);
 	exit(EXIT_FAILURE);
-    }
-#ifdef PTYS_ARE_PTMX
+      }
+    } else
+      fd=rxvt_ttyfd;
+
+#if defined(PTYS_ARE_PTMX) && defined(I_PUSH)
 /*
  * Push STREAMS modules:
  *    ptem: pseudo-terminal hardware emulation module.
  *    ldterm: standard terminal line discipline.
  *    ttcompat: V7, 4BSD and XENIX STREAMS compatibility module.
  */
-    if (!changettyowner) {
-	ioctl(fd, I_PUSH, "ptem");
-	ioctl(fd, I_PUSH, "ldterm");
-	ioctl(fd, I_PUSH, "ttcompat");
+# ifdef HAVE_ISASTREAM
+    if (isastream(fd) == 1)
+# endif
+    {
+        ioctl(fd, I_PUSH, "ptem");
+        ioctl(fd, I_PUSH, "ldterm");
+        ioctl(fd, I_PUSH, "ttcompat");
     }
 #endif
+
     if (changettyowner) {
     /* change ownership of tty to real uid and real group */
 	unsigned int    mode = 0622;
