@@ -15,6 +15,12 @@
 
 #include "xdndclass.h"
 
+//Bitmaps and Pixmaps
+#include "dnd1.xbm"
+#include "dnd1m.xbm"
+#include "dnd2.xbm"
+#include "dnd2m.xbm"
+
 DndManager* default_dnd_man=0;
 
 /////////////////////////////////DndObject/////////////////////////////////////
@@ -45,6 +51,12 @@ int DndObject::dnd_drop(DndData* d, int x, int y,Atom action)
   return 1;
 }
 
+int DndObject::dnd_drag_now(DndData* dat)
+{
+  
+  return dndman->dragging(this,dat);
+}
+
 //////////////////////////////////DndData//////////////////////////////////////
 
 int  DndData::calc_entries(char* dat, char sym)
@@ -59,6 +71,22 @@ int  DndData::calc_entries(char* dat, char sym)
   return i;
 }
 
+void DndData::add_file_entry(char *dir, char* fname)
+{
+  while(*dir)
+  {
+    data[dat_idx++]=*dir++;
+  }
+  if(*(dir-1)!='/')
+    data[dat_idx++]='/';
+  while(*fname)
+  {
+    data[dat_idx++]=*fname++;
+  }
+  data[dat_idx++]='\n';
+  data[dat_idx]=0;
+}
+
 void DndData::free_file_list()
 {
   int i;
@@ -200,11 +228,38 @@ DndManager::DndManager(Display* d,Window irootw)
 
   make_dnd_aware();
 
+  create_dnd_cursors();
+
   xncdprintf(("DndManager initialized. DndEnter is 0x%x\n",XdndEnter));
   //  add_in_type("text/plain");
   //add_in_type("text/uri-list");
 }
 
+void DndManager::create_dnd_cursors()
+{
+  Pixmap dnd1pix,dnd1mpix;
+  XColor xc1, xc2;
+  xc1.pixel = BlackPixel(disp,DefaultScreen(disp));
+  xc1.red = xc1.green = xc1.blue = 0l;
+  xc1.flags = DoRed | DoBlue | DoGreen;
+  xc2.pixel = WhitePixel(disp,DefaultScreen(disp));
+  xc2.red = 65535l;
+  xc2.green = 65535l;
+  xc2.blue = 65535l;
+  xc2.flags = DoRed | DoBlue | DoGreen;
+  dnd1pix = XCreateBitmapFromData(disp, rootw, (char *)dnd1_bits, dnd1_width, dnd1_height);
+  dnd1mpix = XCreateBitmapFromData(disp, rootw, (char *)dnd1m_bits, dnd1m_width, dnd1m_height);
+  curs_one = XCreatePixmapCursor(disp, dnd1pix, dnd1mpix, &xc1, &xc2, 11, 8);
+  XFreePixmap(disp, dnd1pix);
+  XFreePixmap(disp, dnd1mpix);
+  dnd1pix = XCreateBitmapFromData(disp, rootw, (char *)dnd2_bits, dnd2_width, dnd2_height);
+  dnd1mpix = XCreateBitmapFromData(disp, rootw, (char *)dnd2m_bits, dnd2m_width, dnd2m_height);
+  curs_many = XCreatePixmapCursor(disp, dnd1pix, dnd1mpix, &xc1, &xc2, 12, 13);
+  XFreePixmap(disp, dnd1pix);
+  XFreePixmap(disp, dnd1mpix);
+}
+
+
 void DndManager::make_dnd_aware()
 {
   XWindowAttributes attr;
@@ -607,6 +662,7 @@ int DndManager::data_ready_get_it()
     send_finished();
   }
   XFree(data);
+  return 1;
 }
 
 //Building list of types from xclient data structure
@@ -714,3 +770,133 @@ void DndManager::send_event(XEvent& ev)
 {
   XSendEvent(disp,srcwin,False,0L,&ev);
 }
+
+int DndManager::is_dnd_aware(DndWin* dndw, Window window)
+{
+    Atom actual;
+    int format;
+    unsigned long count, remaining;
+    unsigned char *data = 0;
+    Atom *types, *t;
+    int result = 1;
+
+    version_in = 0;
+    XGetWindowProperty (disp, window, XdndAware,
+			0, 0x8000000L, False, XA_ATOM,
+			&actual, &format,
+			&count, &remaining, &data);
+    
+    if (actual != XA_ATOM || format != 32 || count == 0 || !data) {
+      xncdprintf(("XGetWindowProperty failed in xdnd_is_dnd_aware - XdndAware = %ld", XdndAware));
+      if (data)
+	XFree (data);
+      return 0;
+    }
+    types = (Atom *) data;
+#if XDND_VERSION >= 3
+    if (types[0] < 3) {
+      if (data)
+	XFree (data);
+      return 0;
+    }
+#endif
+    version_in = XDND_VERSION < types[0] ? XDND_VERSION : types[0];	/* minimum */
+    xncdprintf(("Using XDND version %d\n", version));
+    if (count > 1) {
+      result = 0;
+      for (t = dndw->typelist; *t; t++) 
+      {
+	int j;
+	for (j = 1; j < count; j++) 
+	{
+	  if (types[j] == *t) 
+	  {
+	    result = 1;
+	    break;
+	  }
+	}
+	if (result)
+	  break;
+      }
+    }
+    XFree (data);
+    return result;
+}
+
+int DndManager::dragging(DndObject* o,DndData* dat)
+{
+  XEvent xevent,xevent_temp;
+  int dnd_aware=0;
+  Window dropper_toplevel;
+
+  xncdprintf(("Starting drag from object %x\n",o));
+  if (XGrabPointer (disp, rootw, False,
+		    ButtonMotionMask | PointerMotionMask | ButtonPressMask 
+		    | ButtonReleaseMask,
+		    GrabModeAsync, GrabModeAsync, None,
+		    curs_one, CurrentTime) != GrabSuccess)
+  {
+    xncdprintf(("Unable to grab pointer"));
+    return 0;
+  }
+
+  while (xevent.xany.type != ButtonRelease) 
+  {
+    XAllowEvents (disp, SyncPointer, CurrentTime);
+    XNextEvent (disp, &xevent);
+    switch (xevent.type) 
+    {
+    case Expose:
+      break;
+    case EnterNotify:
+      /* this event is not actually reported, so we find out by ourselves from motion events */
+      break;
+    case LeaveNotify:
+      /* this event is not actually reported, so we find out by ourselves from motion events */
+      break;
+    case ButtonRelease:
+      /* done, but must send a leave event */
+      break;
+    case MotionNotify:
+      dnd_aware = 0;
+      memcpy (&xevent_temp, &xevent, sizeof (xevent));
+      xevent.xmotion.subwindow = xevent.xmotion.window;
+      {
+	Window root_return, child_return;
+	int x_temp, y_temp;
+	unsigned int mask_return;
+	while (XQueryPointer (disp, xevent.xmotion.subwindow, 
+			      &root_return, &child_return,
+			      &x_temp, &y_temp, &xevent.xmotion.x,
+			      &xevent.xmotion.y, &mask_return)) 
+	{
+#if XDND_VERSION >= 3
+	  if (!dnd_aware) 
+	  {
+	    if ((dnd_aware = is_dnd_aware(o->get_dndwin(), xevent.xmotion.subwindow))) 
+	    {
+	      dropper_toplevel = xevent.xmotion.subwindow;
+	      xevent.xmotion.x_root = x_temp;
+	      xevent.xmotion.y_root = y_temp;
+	      xncdprintf(("Window 0x%x is supported XDND version %d\n",dropper_toplevel,version_in));
+	    }
+	  }
+
+	  dnd_aware=0;
+#else
+	  xevent.xmotion.x_root = x_temp;
+	  xevent.xmotion.y_root = y_temp;
+#endif
+	  if (!child_return)
+	    goto found_descendent;
+	  xevent.xmotion.subwindow = child_return;
+	}
+	break;
+      }
+    found_descendent:
+      break;
+    }
+  }
+  XUngrabPointer(disp,CurrentTime);
+  return 1;
+}
